---
-- Contains definitions for the LDB data source.
-- @file XToLevel.Display.lua
-- @release 3.3.3_13r
-- @copyright Atli Þór (atli@advefir.com)
---
--module "XToLevel.Tooltip" -- For documentation purposes. Do not uncomment!

XToLevel.LDB = 
{
    -- Constants
    textPatterns = {
        default = "{kills}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/kills}{quests}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/quests}{dungeons}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/dungeons}{bgs}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/bgs}{bgo}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/bgo}{xp}{$seperator: }{progress}[{$value}]{/progress}{/xp}{pet}{$seperator: }{color=cfcfdf}{$label}:{/color} {progress}{$value}{/progress}{/pet}{petxp}{$seperator: }{progress}[{$value}]{/progress}{/petxp}",
        minimal = "{kills}{progress}{$value}{/progress}{/kills}{quests}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/quests}{dungeons}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/dungeons}{bgs}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/bgs}{xp}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/xp}{pet}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/pet}{petxp}{color=cfcfdf}{$seperator:/}{/color}{progress}{$value}{/progress}{/petxp}",
        minimal_dashed = "{kills}{progress}{$value}{/progress}{/kills}{quests}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/quests}{dungeons}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/dungeons}{bgs}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/bgs}{xp}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/xp}{pet}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/pet}{petxp}{color=cfcfdf}{$seperator:-}{/color}{progress}{$value}{/progress}{/petxp}",
        brackets = "{kills}{progress}[{$value}]{/progress}{/kills}{quests}{progress}[{$value}]{/progress}{/quests}{dungeons}{progress}[{$value}]{/progress}{/dungeons}{bgs}{progress}[{$value}]{/progress}{/bgs}{xp}{progress}[{$value}]{/progress}{/xp}{pet}{$seperator:-}{progress}[{$value}]{/progress}{/pet}{petxp}{progress}[{$value}]{/progress}{/petxp}",
    },
    textTags = {
        [1] = { tag = "kills", label = 'init', value = '~', color = nil, },
        [2] = { tag = "quests", label = 'init', value = '~', color = nil, },
        [3] = { tag = "dungeons", label = 'init', value = '~', color = nil, }, 
        [4] = { tag = "bgs", label = 'init', value = '~', color = nil, }, 
        [5] = { tag = "bgo", label = 'init', value = '~', color = nil, }, 
        [6] = { tag = "xp", label = 'init', value = '~', color = nil, },
        [7] = { tag = "pet", label = 'init', value = '~', color = nil, },
        [8] = { tag = "petxp", label = 'init', value = '~', color = nil, },
    },

    -- Members
    dataObject = nil,
    mouseOver = false,
    currentPattern = nil,
        
    ---
    -- Constructor
    Initialize = function(self)
        -- Initialize the data object
        local iconName = (UnitFactionGroup("player") == "Alliance") and "INV_Jewelry_TrinketPVP_01" or "INV_Jewelry_TrinketPVP_02"
        local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
        self.dataObject = ldb:NewDataObject("XToLevel", {
            type = "data source",
            icon = "Interface\\Icons\\" .. iconName,
            text = "XToLevel",
            label = "XToLevel",
            version = "3.3.2_13r",
            align = "right",
            ["X-Category"] = "Information"
        });
        ldb = nil;
        
        -- Set data object events
        function self.dataObject:OnEnter()
            XToLevel.LDB.mouseOver = true;
            local a1, f1, a2 = FindAnchor(self);
            XToLevel.Tooltip:Show(self, a1, f1, a2, L['Click To Configure']);
        end
        function self.dataObject:OnLeave()
            XToLevel.LDB.mouseOver = false;
            XToLevel.Tooltip:Hide()
        end
        function self.dataObject:OnClick(button)
			XToLevel.Config:Open("ldb")
        end
        
        self:BuildPattern();-- /run XToLevel.LDB:BuildPattern(); XToLevel.LDB:Update();
        self:Update();
    end,
        
    ---
    -- Creates the text pattern to be used.
    BuildPattern = function(self)
        if ((XToLevel.Player.level < XToLevel.Player.maxLevel) or XToLevel.Pet.isActive or XToLevel.Pet.hasBeenActive) then
            local showPlayer, showPet, playerProgress, playerProgressColor, petProgress, petProgressColor;
            local newText, useColors, isFirst, sPos, ePost, value, rest, attributes, out;
        
            showPlayer = XToLevel.Player.isActive and (XToLevel.Player.level < XToLevel.Player.maxLevel)
            showPet = XToLevel.Pet.isActive or XToLevel.Pet.hasBeenActive
            
            -- User is leveling, proceed normally.
            if showPlayer then
                playerProgress = round(XToLevel.Player.currentXP / XToLevel.Player.maxXP * 100, 1)
                playerProgressColor = GetProgressColor_Soft(playerProgress)
            end
                
            if showPet then
                petProgress = round(XToLevel.Pet.xp / XToLevel.Pet.maxXP * 100, 1)
                petProgressColor = GetProgressColor_Soft(petProgress)
            end
            
            -- Load the appropriate pattern, attempting to find one in the config.
            newText = self.textPatterns.default
            if sConfig.ldb.textPattern ~= nil then
                if sConfig.ldb.textPattern == "custom" then
                    newText = sData.customPattern or "Please choose a pattern."
                    
                    -- Parse html-like syntax
                    newText = string.gsub(newText, '<(%w+) ?(.-)>', function(tag, attr)
                        attributes = {
                            label = false,
                            post = false,
                            seperator = false,
                        }
                        
                        for key, value in pairs(attributes) do
                            sPos, ePost, value, rest = string.find(attr, key ..'="([^"]-)"')
                            if value ~= nil then
                                attributes[key] = value
                            end
                        end
                        
                        out = "{".. tag .."}"
                        if attributes.seperator then
                            out = out .. "{$seperator:" .. attributes.seperator .."}"
                        end
                        if attributes.label then
                            out = out .. attributes.label ..": "
                        end
                        out = out .. "{progress}{$value}"
                        if attributes.post then
                            out = out .. attributes.post or nil
                        end
                        out = out .. "{/progress}{/" .. tag .."}"
                        return out
                    end)
                else
                    newText = self.textPatterns[sConfig.ldb.textPattern]
                end
            end
            
            -- Replace {color} tags
            if sConfig.ldb.allowTextColor then 
                newText = string.gsub(newText, "{color=([0-9A-Fa-f]+)}(.-){/color}", "|cFF%1%2|r")
            else
                newText = string.gsub(newText, "{color=[0-9A-Fa-f]+}(.-){/color}", "%1")
            end
            
            -- Prepare tags
            useColors = (sConfig.ldb.allowTextColor and sConfig.ldb.text.colorValues)
            self.textTags = {
                [1] = {
                    tag = "kills",
                    label = (sConfig.ldb.text.verbose and L["Kills"] ) or L["Kills Short"],
                    value = (showPlayer and '$$kills$$') or nil,
                    color = (useColors and '$$playercolor$$') or nil,
                },
                [2] = { 
                    tag = "quests",
                    label = (sConfig.ldb.text.verbose and L["Quests"] ) or L["Quests Short"],
                    value = (showPlayer and '$$quests$$') or nil,
                    color = (useColors and '$$playercolor$$') or nil,
                }, 
                [3] = { 
                    tag = "dungeons",
                    label = (sConfig.ldb.text.verbose and L["Dungeons"] ) or L["Dungeons Short"],
                    value = ((showPlayer and UnitLevel("Player") >= 15) and '$$dungeons$$') or nil,
                    color = (useColors and '$$playercolor$$') or nil,
                }, 
                [4] = { 
                    tag = "bgs",
                    label = (sConfig.ldb.text.verbose and L["Battles"] ) or L["Battles Short"],
                    value = ((showPlayer and UnitLevel("Player") >= 10) and '$$bgs$$') or nil,
                    color = (useColors and '$$playercolor$$') or nil,
                }, 
                [5] = { 
                    tag = "bgo",
                    label = (sConfig.ldb.text.verbose and L["Objectives"] ) or L["Objectives Short"],
                    value = ((showPlayer and UnitLevel("Player") >= 10) and '$$bgo$$') or nil,
                    color = (useColors and '$$playercolor$$') or nil,
                }, 
                [6] = { 
                    tag = "xp",
                    label = L["XP"],
                    value = (showPlayer and ('$$xp$$')) or nil,
                    color = (sConfig.ldb.allowTextColor and '$$playercolor$$') or nil,
                },
                [7] = { 
                    tag = "pet",
                    label = (sConfig.ldb.text.verbose and L["Pet"] ) or L["Pet Short"],
                    value = (showPet and '$$pet$$') or nil,
                    color = (useColors and '$$petcolor$$') or nil,
                },
                [8] = { 
                    tag = "petxp",
                    label = L["XP"],
                    value = (showPet and '$$petxp$$') or nil,
                    color = (sConfig.ldb.allowTextColor and '$$petcolor$$') or nil,
                },
            }
            
            -- Replace values
            isFirst = true
            for i, object in ipairs(self.textTags) do
                if sConfig.ldb.text[object.tag] and object.value ~= nil then
                    newText = string.gsub(newText, "{".. object.tag .."}(.-){/".. object.tag .."}", function(str)
                        str = string.gsub(str, "{$label}", object.label)
                        str = string.gsub(str, "{$value}", object.value)
                        if object.color ~= nil then
                            str = string.gsub(str, "{progress}(.-){/progress}", "|cFF".. object.color .."%1|r")
                        else
                            str = string.gsub(str, "{progress}(.-){/progress}", "%1")
                        end
                        if isFirst then
                            str = string.gsub(str, "{$seperator(.-)}", "")
                        else
                            str = string.gsub(str, "{$seperator:?(.-)}", "%1")
                        end
                        return str
                    end)
                    isFirst = false
                else
                    newText = string.gsub(newText, "{".. object.tag .."}.-{/".. object.tag .."}", "")
                end
            end
            self.currentPattern = newText
            
            -- Free resources
            showPlayer = nil;
            showPet = nil;
            playerProgress = nil;
            playerProgressColor = nil;
            petProgress = nil;
            petProgressColor = nil;
            newText = nil;
            useColors = nil;
            isFirst = nil;
            sPos = nil;
            ePost = nil;
            value = nil;
            rest = nil;
            attributes = nil;
            out = nil;
            
        else
            -- Player is at max level and has no pets.
            if sConfig.ldb.customColors then
                self.currentPattern = "|cFFaaaaaaInactive|r"
            else
                self.currentPattern = "Inactive"
            end
        end
    end,
        
    ---
    -- Update LDB text
    -- Note, this is a very memory demanding function.
    --  Calling it periodically is not a good idea!
    Update = function(self)
        if self.dataObject == nil then
            return false;
        end
        
        if sConfig.ldb.showLabel then
            self.dataObject.label = L["XToLevel"]
        else
            self.dataObject.label = nil
        end
            
        if sConfig.ldb.showIcon then
            local iconName = (UnitFactionGroup("player") == "Alliance") and "INV_Jewelry_TrinketPVP_01" or "INV_Jewelry_TrinketPVP_02"
            self.dataObject.icon = "Interface\\Icons\\" .. iconName
        else
            self.dataObject.icon = nil
        end
        
        if sConfig.ldb.showText then
            local pattern = self.currentPattern;
            if (XToLevel.Player.level < XToLevel.Player.maxLevel) then
                local playerProgress = XToLevel.Player:GetProgressAsPercentage(0)
                local playerProgressColor = GetProgressColor_Soft(playerProgress)
                
                pattern = string.gsub(pattern, '%$%$playercolor%$%$', playerProgressColor);
                pattern = string.gsub(pattern, '%$%$kills%$%$', (round(XToLevel.Player:GetAverageKillsRemaining()) or "~"));
                pattern = string.gsub(pattern, '%$%$quests%$%$', (round(XToLevel.Player:GetAverageQuestsRemaining()) or "~"));
                pattern = string.gsub(pattern, '%$%$dungeons%$%$', (round(XToLevel.Player:GetAverageDungeonsRemaining()) or "~"));
                if sConfig.ldb.text.xpAsBars then
                    pattern = string.gsub(pattern, '%$%$xp%$%$', tostring(XToLevel.Player:GetProgressAsBars()) .. " " .. L['Bars']);
                else
                    pattern = string.gsub(pattern, '%$%$xp%$%$', playerProgress .. "%%");
                end
                pattern = string.gsub(pattern, '%$%$bgs%$%$', (round(XToLevel.Player:GetAverageBGsRemaining()) or "~"));
                pattern = string.gsub(pattern, '%$%$bgo%$%$', (round(XToLevel.Player:GetAverageBGObjectivesRemaining()) or "~"));
            else
                pattern = string.gsub(pattern, '%$%$playercolor%$%$', '');
                pattern = string.gsub(pattern, '%$%$kills%$%$', '');
                pattern = string.gsub(pattern, '%$%$quests%$%$', '');
                pattern = string.gsub(pattern, '%$%$dungeons%$%$', '');
                pattern = string.gsub(pattern, '%$%$xp%$%$', '');
                pattern = string.gsub(pattern, '%$%$bgs%$%$', '');
                pattern = string.gsub(pattern, '%$%$bgo%$%$', '');
            end
                
            if XToLevel.Pet.isActive or XToLevel.Pet.hasBeenActive then
                local petProgress, petProgressColor;
                petProgress = round(XToLevel.Pet.xp / XToLevel.Pet.maxXP * 100, 1)
                petProgressColor = GetProgressColor_Soft(petProgress)
                
                pattern = string.gsub(pattern, '%$%$petcolor%$%$', petProgressColor);
                pattern = string.gsub(pattern, '%$%$pet%$%$', (round(XToLevel.Pet:GetAverageKillsRemaining()) or "~"));
                if sConfig.ldb.text.xpAsBars then
                    pattern = string.gsub(pattern, '%$%$petxp%$%$', tostring(XToLevel.Pet:GetProgressAsBars()) .. " " .. L['Bars']);
                else
                    pattern = string.gsub(pattern, '%$%$petxp%$%$', petProgress .. "%%");
                end
            else
                pattern = string.gsub(pattern, '%$%$petcolor%$%$', '');
                pattern = string.gsub(pattern, '%$%$pet%$%$', '');
                pattern = string.gsub(pattern, '%$%$petxp%$%$', '');
            end
            self.dataObject.text = pattern;
        else
            self.dataObject.text = nil
        end
    end
}